==> otobo-10.0.12.tar.gz: otobo-10.0.12/Kernel/System/MigrateFromOTRS.pm <==
# --
# OTOBO is a web-based ticketing system for service organisations.
# --
# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
# Copyright (C) 2019-2021 Rother OSS GmbH, https://otobo.de/
# --
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
# --

package Kernel::System::MigrateFromOTRS;

use strict;
use warnings;
use v5.24;
use namespace::autoclean;
use utf8;

# core modules

# CPAN modules

# OTOBO modules

our @ObjectDependencies = (
    'Kernel::System::Main',
    'Kernel::System::MigrateFromOTRS::Base',
    'Kernel::System::Log',
);

=head1 NAME

Kernel::System::MigrateFromOTRS - Perform system migration from OTRS 6 to OTOBO 10.

=head1 PUBLIC INTERFACE

=head2 new()

Don't use the constructor directly, use the ObjectManager instead:

    my $MigrateFromOTRSObject = $Kernel::OM->Get('Kernel::System::MigrateFromOTRS');

=cut

sub new {
    my ( $Type, %Param ) = @_;

    # allocate new hash for object
    return bless {}, $Type;
}

=head2 Run()

run migration task

    my $Success = $MigrateFromOTRSObject->Run (
        Task          => 'All' # or only one Task
        UserID        => 123,
    );

=cut

sub Run {
    my ( $Self, %Param ) = @_;

    # check needed stuff
    for my $Needed (qw(Task UserID)) {
        if ( !$Param{$Needed} ) {
            $Kernel::OM->Get('Kernel::System::Log')->Log(
                Priority => 'error',
                Message  => "Need $Needed!",
            );

            return;
        }
    }

    # don't attempt to run when the pre check failed
    return unless $Self->_ExecutePreCheck(%Param);

    return $Self->_ExecuteRun(%Param);
}

sub _ExecutePreCheck {
    my ( $Self, %Param ) = @_;

    # determine the relevant tasks
    my @Tasks;
    {
        my @AllTasks = grep { $_ && $_->{Module} } $Self->_TasksGet();

        if ( $Param{Task} eq 'All' ) {
            @Tasks = @AllTasks;
        }
        else {
            @Tasks = grep { $_->{Module} eq $Param{Task} } @AllTasks;
        }
    }

    if ( !@Tasks ) {
        print STDERR "No valid Module $Param{Task} found. ",
            q{Perhaps you need to add the new check to $Self->_TasksGet().};

        return 0;
    }

    my $IsOK = 0;

    TASK:
    for my $Task (@Tasks) {

        my $ModuleName = "Kernel::System::MigrateFromOTRS::$Task->{Module}";

        # NOTE: This looks strange. When one check succeeded and next check can't be loaded that the
        # check succeeds altogether.
        last TASK unless $Kernel::OM->Get('Kernel::System::Main')->Require($ModuleName);

        # NOTE: see the note above
        $Self->{TaskObjects}->{$ModuleName} //= $Kernel::OM->Create($ModuleName);

        last TASK unless $Self->{TaskObjects}->{$ModuleName};

        # Execute previous check, printing a different message
        # NOTE: when last check succeeds the the check succeeds altogether
        if ( $Self->{TaskObjects}->{$ModuleName}->can('CheckPreviousRequirement') ) {
            $IsOK = $Self->{TaskObjects}->{$ModuleName}->CheckPreviousRequirement(%Param);
        }
    }

    return $IsOK;
}

sub _ExecuteRun {
    my ( $Self, %Param ) = @_;

    # determine the relevant tasks
    my @Tasks;
    {
        my @AllTasks = grep { $_ && $_->{Module} } $Self->_TasksGet();

        if ( $Param{Task} eq 'All' ) {
            @Tasks = @AllTasks;
        }
        else {
            @Tasks = grep { $_->{Module} eq $Param{Task} } @AllTasks;
        }
    }

    if ( !@Tasks ) {

        return {
            Message => "invalid task $Param{Task}",
            Comment => "No valid Module $Param{Task} found. "
                . qq{Perhaps you need to add the new check to $Self->_TasksGet().},
            Successful => 0,
        };
    }

    my $CurrentStep = 1;
    my %Result;

    TASK:
    for my $Task (@Tasks) {

        my $ModuleName = "Kernel::System::MigrateFromOTRS::$Task->{Module}";

        last TASK unless $Kernel::OM->Get('Kernel::System::Main')->Require($ModuleName);

        $Self->{TaskObjects}->{$ModuleName} //= $Kernel::OM->Create($ModuleName);

        last TASK unless $Self->{TaskObjects}->{$ModuleName};

        # Execute Run-Component
        if ( $Self->{TaskObjects}->{$ModuleName}->can('Run') ) {

            $Result{ $Task->{Module} } = $Self->{TaskObjects}->{$ModuleName}->Run(%Param);

            # Add counter to $Result.
            $Result{ $Task->{Module} }->{CurrentStep} = $CurrentStep;
        }

        # Do not handle Run if task has no appropriate method.
        else {
            next TASK;
        }

        $CurrentStep++;
    }

    return \%Result;
}

sub _TasksGet {
    my ( $Self, %Param ) = @_;

    my $MigrationBaseObject = $Kernel::OM->Get('Kernel::System::MigrateFromOTRS::Base');

    return $MigrationBaseObject->TaskSecurityCheck();
}

1;

==> otobo-10.0.20.tar.gz: otobo-10.0.20/Kernel/System/MigrateFromOTRS.pm <==
# --
# OTOBO is a web-based ticketing system for service organisations.
# --
# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
# Copyright (C) 2019-2024 Rother OSS GmbH, https://otobo.de/
# --
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
# --

package Kernel::System::MigrateFromOTRS;

use strict;
use warnings;
use v5.24;
use namespace::autoclean;
use utf8;

# core modules

# CPAN modules

# OTOBO modules

our @ObjectDependencies = (
    'Kernel::System::Main',
    'Kernel::System::Log',
);

=head1 NAME

Kernel::System::MigrateFromOTRS - Perform system migration from OTRS 6 to OTOBO 10.

=head1 PUBLIC INTERFACE

=head2 new()

Don't use the constructor directly, use the ObjectManager instead:

    my $MigrateFromOTRSObject = $Kernel::OM->Get('Kernel::System::MigrateFromOTRS');

=cut

sub new {
    my ( $Type, %Param ) = @_;

    # allocate new hash for object
    return bless {}, $Type;
}

=head2 Run()

run migration task

    my $Success = $MigrateFromOTRSObject->Run (
        Task          => 'All' # or only one Task
        UserID        => 123,
    );

=cut

sub Run {
    my ( $Self, %Param ) = @_;

    my %ResultFail = ( Successful => 0 );

    # check needed stuff
    for my $Needed (qw(Task UserID)) {
        if ( !$Param{$Needed} ) {
            $Kernel::OM->Get('Kernel::System::Log')->Log(
                Priority => 'error',
                Message  => "Need $Needed!",
            );

            return \%ResultFail;
        }
    }

    my $Task       = $Param{Task};
    my $ModuleName = "Kernel::System::MigrateFromOTRS::$Task";

    return \%ResultFail unless $Kernel::OM->Get('Kernel::System::Main')->Require($ModuleName);

    my $TaskObject = $Kernel::OM->Create($ModuleName);

    return \%ResultFail unless $TaskObject;

    # don't attempt to run when the pre check failed
    return \%ResultFail unless $Self->_ExecutePreCheck( %Param, TaskObject => $TaskObject );

    return $Self->_ExecuteRun( %Param, TaskObject => $TaskObject );
}

sub _ExecutePreCheck {
    my ( $Self, %Param ) = @_;

    my $TaskObject = delete $Param{TaskObject};

    # successful per default, CheckPreviousRequirement() is not a required method
    return 1 unless $TaskObject->can('CheckPreviousRequirement');

    return $TaskObject->CheckPreviousRequirement(%Param);
}

sub _ExecuteRun {
    my ( $Self, %Param ) = @_;

    my $TaskObject = delete $Param{TaskObject};

    my %ResultFail = ( Successful => 0 );

    # a migration step must have a Run-method
    return \%ResultFail unless $TaskObject->can('Run');

    # Execute Run-Component
    return $TaskObject->Run(%Param);
}

1;
